recoveryButtonName = "復元";
errorMessageAutoSave =
  "自動保存機能にエラーが起きました。\n※シートの登録・変更には問題ありません。";
errorMessageRecovery =
  "復元する際にエラーが起きました。\n※シートの登録・変更には問題ありません。";
willRecovery = "時点の「詳細内容」が保存されています。復元しますか?";

//IEか判断する変数
const isIE = (function () {
  //閲覧しているブラウザの情報（ユーザーエージェント）を取得
  var userAgent = window.navigator.userAgent.toLocaleLowerCase();

  //IE判定(IE11とIE10以前)、アラート表示。
  if (userAgent.indexOf("trident") != -1) {
    return true;
  }
  return false;
})();

/**
 * 復元ボタンを生成する
 *
 * @param {*} buttonName 生成するボタンの名前
 * @param {*} addDomLocation ボタンを生成するDOMをjQueryで渡す
 */
function createButton(buttonName, addDomLocation) {
  //li要素の生成
  //jQueryで書き換え
  const $li = $("<li>").attr({
    id: "addListRecovery",
  });

  const $a = $("<a>")
    .attr({
      id: "recovery",
      class: "btn",
      href: "javascript:void(0)",
    })
    .text(buttonName);

  addDomLocation.prepend($li);
  $("#addListRecovery").prepend($a);
  $("#recovery").css({ color: "#ffffff", backgroundColor: "#009900" });
}

/**
 * eStorageないのsaveContentに今回の案件コードと同じものがあるか判断
 *
 * @param searchObj 検索したいオブジェクトの配列
 * @param searchCode 検索したい数値
 * @returns boolean 渡された配列の中に対象のコードがあるか判断してある場合はtrue、ない場合はfalse
 */
function backupExists(searchObj, searchCode) {
  //eStorageないのsaveContentに今回の案件コードと同じものがあるか判断
  for (let i = 0; i < searchObj.length; i++) {
    if (searchObj[i].id == searchCode) {
      return true;
    }
  }
  return false;
}

/**
 * searchObj内にあるsearchCodeのインデックス番号を返す。
 * ヒットしない場合は0を返す。
 *
 * @param searchObj 検索したいオブジェクトの配列
 * @param searchCode 検索したい数値
 * @returns int ローカルストレージにある対象のインデックス番号を返す
 */
function getBackupId(searchObj, searchCode) {
  //eStorageないのsaveContentに今回の案件コードと同じものがあるか判断
  for (let i = 0; i < searchObj.length; i++) {
    if (searchObj[i].id == searchCode) {
      return i;
    }
  }
  return 0;
}

/**
 * searchObj内にある日付が一番古いもののインデックスを返す
 * 空白の場合はそこでインデックスを返す
 *
 * @param searchObj 検索したいオブジェクト
 * @returns int ヒットしたインデックス番号
 */
function getOldDateIndexNum(searchObj) {
  //saveContentの1つめがcreateDateの中身が空白ならtargetNumにセットする
  if (searchObj[0].createDate == "") {
    return 0;
  }

  //一番過去の日付の配列を求めるために必要。
  let min = searchObj[0].createDate;
  let minId = 0;

  //古い保存内容のインデックス番号をtargetNumにセットする
  for (let i = 0; i < searchObj.length; i++) {
    //saveContentの1つめがcreateDateの中身が空白ならtargetNumにセットする
    if (searchObj[i].createDate == "") {
      return i;
    }
    //保存内容が古いインデックス番号を保存
    if (searchObj[i].createDate < min) {
      min = searchObj[i].createDate;
      minId = i;
    }
  }
  return minId;
}

$(document).ready(function () {
  if (isIE) {
    return;
  }
  if ($('textarea[name="note"]').length == 0) {
    return;
  }
  if ($('ul[class="tools"]').length == 0) {
    return;
  }
  try {
    //活動に紐づく案件のコードをもらう
    const companyCode = $("input[name=company_code]")[0].value;
    //活動に紐づく案件のコードをもらう
    const salesCode = $("input[name=sales_code]")[0].value;
    //コンテンツコード
    const saveContentId = `${companyCode}_${salesCode}`;
    //保存できる記入情報の数
    const MaxInputNum = 5;
    //e-storageがある判定
    const isEStorage = localStorage.getItem("eStorage");
    //saveContentの中にidが案件コードと同じものはあるか?
    let targetNum = 0;
    //ローカルストレージの中身を保存する
    let eStorage = {};
    //詳細内容のみのDOM情報
    const $note = $('textarea[name="note"]');
    //前回入力情報がある場合に日付を保存する
    let after_input_date = "";

    if (isEStorage) {
      eStorage = JSON.parse(isEStorage);
    } else {
      //eStorageを生成
      const content = [...Array(MaxInputNum).keys()].map(function (val) {
        return {
          id: val,
          createDate: "",
          backupContent: {},
        };
      });
      eStorage = { saveContent: content };
      localStorage.setItem("eStorage", JSON.stringify(eStorage));
    }

    //前回の入力情報があるか判断する
    const isBackup = backupExists(eStorage.saveContent, saveContentId);

    //前回記入情報がない場合はeStorage.saveContentのどこに保存するか決める
    if (isBackup) {
      targetNum = getBackupId(eStorage.saveContent, saveContentId);
      //前回入力情報がある場合は前回の日付を保存する。
      after_input_date = new Date(eStorage.saveContent[targetNum].createDate);
      //詳細内容と復元内容が同じ場合は復元ボタンを出さない
      if (
        $note[0].value != eStorage.saveContent[targetNum].backupContent.note
      ) {
        createButton(recoveryButtonName, $('ul[class="tools"]'));
      }
    } else {
      //日付の一番古い所にバックアップする
      targetNum = getOldDateIndexNum(eStorage.saveContent);
      //上書きする方の中身を初期化する
      const target = eStorage.saveContent[targetNum];
      target.id = saveContentId;
      target.backupContent = {};
    }

    //$noteの変更を感知した時
    $note.change(function (event) {
      try {
        //フォームの書き換えデータと過去の書き換えデータを持つ
        //変更した場所のDOM情報を渡す
        const $changed_dom = $(event.target);
        //書き換えのあったDOMのnameを取得
        const keyName = $changed_dom.attr("name");
        //書き換えのあったDOMのvalueを取得
        const value = $changed_dom.val();
        //書き換えのあった時の日時を保存
        eStorage.saveContent[targetNum].createDate = Date.now();
        //backupContentにキー値；書き換えのあったDOMのname名で値を設定
        eStorage.saveContent[targetNum].backupContent[keyName] = value;
        //表示する時に必要になる前回記入情報も変更
        after_input_date = new Date(eStorage.saveContent[targetNum].createDate);
        //localStorageに保存
        localStorage.setItem("eStorage", JSON.stringify(eStorage));
      } catch (e) {
        console.error(e.name + ":" + e.message);
        alert(errorMessageAutoSave);
      }
    });

    //復元ボタン押した際にformDataクラスのformDataを引数に復元する
    $("#recovery").on("click", function () {
      try {
        const afterInputDateString =
          after_input_date.getFullYear() +
          "/" +
          (after_input_date.getMonth() + 1) +
          "/" +
          after_input_date.getDate() +
          " " +
          after_input_date.getHours() +
          ":" +
          after_input_date.getMinutes();
        if (window.confirm(afterInputDateString + willRecovery)) {
          //復元内容を画面に描写する
          const value = eStorage.saveContent[targetNum].backupContent.note;
          $note[0].value = value;
          $("#addListRecovery").remove();
        }
      } catch (e) {
        console.error(e.name + ":" + e.message);
        alert(errorMessageRecovery);
      }
    });
  } catch (e) {
    console.error(e.name + ":" + e.message);
  }
});
